@@ -4,8 +4,6 @@ from pycrfsuite_spacing import PyCRFSuiteSpacing
 from pycrfsuite_spacing import TemplateGenerator
 from pycrfsuite_spacing import CharacterFeatureTransformer
 
-from treform.segmentation.cnnWordSegmentationModel import SpacingModel
-
 import tensorflow as tf
 
 
@@ -142,33 +140,25 @@ class CNNSegmentationKorean(BaseSegmentation):
         self.training_config = training_config
         self.char_file = char_file
 
+        #self.model, self.vocab_table = self.load(model_file=model_file, training_config=training_config, char_file=char_file)
+
+
     def load(self, model_file='', training_config='', char_file=''):
-        tf.config.run_functions_eagerly(True)
-
-        print(tf.executing_eagerly())
-
-        with open(self.training_config, encoding='utf-8') as f:
+        with open(training_config, encoding='utf-8') as f:
             config = json.load(f)
 
-        with open(self.char_file, encoding='utf=8') as f:
+        with open(char_file, encoding='utf=8') as f:
             content = f.read()
             keys = ["<pad>", "<s>", "</s>", "<unk>"] + list(content)
             values = list(range(len(keys)))
 
-            vocab_initializer = tf.lookup.KeyValueTensorInitializer(keys, values, key_dtype=tf.string,
-                                                                    value_dtype=tf.int32)
+            vocab_initializer = tf.lookup.KeyValueTensorInitializer(keys, values, key_dtype=tf.string, value_dtype=tf.int32)
             vocab_table = tf.lookup.StaticHashTable(vocab_initializer, default_value=3)
 
-        model = SpacingModel(
-            config["vocab_size"],
-            config["hidden_size"],
-            conv_activation=config["conv_activation"],
-            dense_activation=config["dense_activation"],
-            conv_kernel_and_filter_sizes=config["conv_kernel_and_filter_sizes"],
-            dropout_rate=config["dropout_rate"],
-        )
+        #To be implemented
+        model = None
 
-        model.load_weights(self.model_file)
+        model.load_weights(model_file)
         model(tf.keras.Input([None], dtype=tf.int32))
         model.summary()
 
@@ -177,61 +167,61 @@ class CNNSegmentationKorean(BaseSegmentation):
     def predict(self, model, vocab_table, input_str):
         inference = self.get_inference_fn(model, vocab_table)
         input_str = tf.constant(input_str)
-        result1 = inference(input_str)
-        sess = tf.compat.v1.Session()
-        init = tf.compat.v1.global_variables_initializer()
-        sess.run(init)
-        sess.run(tf.compat.v1.tables_initializer())
-
-        result = sess.run(inference(input_str))
-        print(result)
-
-        print(b"".join(result).decode("utf8"))
+        result = inference(input_str).numpy()
+        #print(b"".join(result).decode("utf8"))
         return b"".join(result).decode("utf8")
 
     def get_inference_fn(self, model, vocab_table):
         @tf.function
         def inference(tensors):
             byte_array = tf.concat(
-                [["<s>"], tf.strings.unicode_split(tf.strings.regex_replace(tensors, " +", " "), "UTF-8"), ["</s>"]],
-                axis=0
+                [["<s>"], tf.strings.unicode_split(tf.strings.regex_replace(tensors, " +", " "), "UTF-8"), ["</s>"]], axis=0
             )
             strings = vocab_table.lookup(byte_array)[tf.newaxis, :]
 
             model_output = tf.argmax(model(strings), axis=-1)[0]
-            return self.convert_output_to_string(byte_array, model_output)
+            return convert_output_to_string(byte_array, model_output)
 
         return inference
 
-    def convert_output_to_string(self, byte_array, model_output):
-        sequence_length = tf.size(model_output)
-        while_condition = lambda i, *_: i < sequence_length
 
-        #@tf.function
-        def while_body(i, o):
-            a = tf.math.equal(model_output[i],2)
-            b = tf.math.equal(byte_array[i],' ')
-            c = tf.math.equal(model_output[i],1)
+    #def __call__(self, *args, **kwargs):
+    #    return self.predict(self.model, self.vocab_table, args[0])
 
-            o = tf.cond(
-                tf.cast(c, tf.bool),
-                lambda: tf.concat([o, [byte_array[i], " "]], axis=0),
-                lambda: tf.cond(
-                    tf.cast(tf.logical_and(a, b),tf.bool),
-                    lambda: o,
-                    lambda: tf.concat([o, [byte_array[i]]], axis=0),
+def convert_output_to_string(byte_array, model_output):
+    sequence_length = tf.size(model_output)
+    while_condition = lambda i, *_: i < sequence_length
+
+    def while_body(i, o):
+        o = tf.cond(
+            model_output[i] == 1,
+            lambda: tf.concat([o, [byte_array[i], " "]], axis=0),
+            lambda: tf.cond(
+                (model_output[i] == 2) and (byte_array[i] == " "),
+                lambda: o,
+                lambda: tf.concat([o, [byte_array[i]]], axis=0),
                 ),
             )
-            return i + 1, o
+        return i + 1, o
 
-        _, strings_result = tf.while_loop(
-            while_condition,
-            while_body,
-            (tf.constant(0), tf.constant([], dtype=tf.string)),
-            shape_invariants=(tf.TensorShape([]), tf.TensorShape([None])),
-        )
-        return strings_result
+    _, strings_result = tf.while_loop(
+        while_condition,
+        while_body,
+        (tf.constant(0), tf.constant([], dtype=tf.string)),
+        shape_invariants=(tf.TensorShape([]), tf.TensorShape([None])),
+    )
+    return strings_result
 
-    def __call__(self, *args, **kwargs):
-        model, vocab_table = self.load(self.model_file, self.training_config, self.char_file)
-        return self.predict(model, vocab_table, args[0])
\ No newline at end of file
+if __name__ == "__main__":
+    model_file = '../../models/checkpoint-12.ckpt'
+    training_config = '../../resources/config.json'
+    char_file = '../../resources/chars-4996'
+
+    spacing = CNNSegmentationKorean()
+
+    model, vocab_table = spacing.load(model_file=model_file, training_config=training_config, char_file=char_file)
+    input_str = '오늘은우울한날이야...ㅜㅜ'
+    result = spacing.predict(model, vocab_table, input_str)
+    result = result.replace('<s>','')
+    result = result.replace('</s>', '')
+    print(result)
\ No newline at end of file
