@@ -1,17 +1,16 @@
 
-import os
-os.environ["TF_CPP_MIN_LOG_LEVEL"] = "3"
+
 import math
 import numpy as np
 
-def iqf_qf_icf(corpus,test,package):
+def iqf_qf_icf(corpus,package):
 	weights = package["weights"]
-	labelset = package["labelset"]
-
 	doclist = {}
 	dictlist = {}
 	worddict = {}
 	word_label_dict = {}
+	labelset = package["labelset"]
+
 	n = len(corpus)
 
 	for i in corpus:
@@ -22,8 +21,9 @@ def iqf_qf_icf(corpus,test,package):
 		if labell not in dictlist:
 			dictlist[labell] = {}
 		if docl not in dictlist[labell]:
-			dictlist[labell][docl] = {}	
-		for j in i["split_sentence"]:
+			dictlist[labell][docl] = {}
+
+		for j in i["split_sentence"].split():
 			#dictlist : label —— doc —— word —— frequency
 			if j not in dictlist[labell][docl]:
 				dictlist[labell][docl][j] = 1
@@ -36,16 +36,6 @@ def iqf_qf_icf(corpus,test,package):
 
 	iqf_qf_icf = {}
 
-	if test == 1:
-		for label in doclist:
-			iqf_qf_icf[label] = {}
-			for word in doclist[label]:
-				if word in weights[label]:
-					iqf_qf_icf[label][word] = weights[label][word]
-				else:
-					iqf_qf_icf[label][word] = max([weights[x][word] for x in weights if word in weights[x]])
-		return iqf_qf_icf
-
 	for labell in labelset:
 		iqf_qf_icf[labell] = {}
 		for word in doclist[labell]:
@@ -67,4 +57,5 @@ def iqf_qf_icf(corpus,test,package):
 	
 	package["weights"] = weights
 	package["labelset"] = labelset
+
 	return weights
